package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import java.util.HashSet;
import java.util.Set;

/**
 * @author mvalko
 */

@Setter
@Getter
@Entity
public class TeamEntity extends BaseEntity {

    @Column(nullable = false, unique = true)
    private String name;
    private CountryEnum country;

    @OneToMany(mappedBy = "team", cascade = CascadeType.PERSIST)
    private Set<PlayerEntity> playerList = new HashSet<>();

    @OneToMany(mappedBy = "team", cascade = CascadeType.PERSIST)
    private Set<TeamCoachEntity> coachList = new HashSet<>();

    @ManyToMany(mappedBy = "teamsEnrolled", cascade = CascadeType.PERSIST)
    private Set<TournamentEntity> completedTournaments = new HashSet<>();

    @ManyToMany(mappedBy = "teamsSigned", cascade = CascadeType.PERSIST)
    private Set<TournamentEntity> plannedTournaments = new HashSet<>();

    @ManyToMany
    private Set<MatchEntity> matches = new HashSet<>();

    public TeamEntity(){

    }

    public TeamEntity(String name, Set<TeamCoachEntity> coaches) {
        this.name = name;
        this.coachList = coaches;
    }

    /**
     * Adds player who recently joined the team to the playerList.
     * @param newPlayer PlayerEntity that is to be added to the playerList.
     */
    public void addPlayerToTheTeam(PlayerEntity newPlayer){
        this.playerList.add(newPlayer);
    }

    /**
     * Removes player who left the team from the playerList.
     * @param oldPlayer PlayerEntity that is to be removed from the playerList.
     */
    public void removePlayerFromTheTeam(PlayerEntity oldPlayer) {
        this.playerList.remove(oldPlayer);
    }

    /**
     * Adds coach who recently joined the team to the coachList.
     * @param newCoach TeamCoachEntity that is to be added to the coachList.
     */
    public void addCoachToTheTeam(TeamCoachEntity newCoach) {
        this.coachList.add(newCoach);
    }

    /**
     * Removes coach who left the team from the coachList.
     * @param oldCoach TeamCoachEntity that is to be removed from the coachList.
     */
    public void removeCoachFromTheTeam(TeamCoachEntity oldCoach) {
        this.coachList.remove(oldCoach);
    }
}
