package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.player.PlayerDao;
import cz.muni.fi.pa165.entity.PlayerEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.testng.AbstractTransactionalTestNGSpringContextTests;
import org.testng.annotations.Test;

import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Marek Kadlečík
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
public class PlayerDaoTest extends AbstractTransactionalTestNGSpringContextTests {

    @Autowired
    private PlayerDao playerDao;


    @Test
    void create() {
        PlayerEntity player = new PlayerEntity("Johan");
        playerDao.create(player);
        assertThat(player.getId()).isNotNull();
        assertThat(playerDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        PlayerEntity player = new PlayerEntity("Adam");
        playerDao.create(player);

        var find = playerDao.findById(player.getId());
        assertThat(find).isEqualTo(player);
    }

    @Test
    void findByIdNonExisting() {
        assertThat(playerDao.findAll()).isEmpty();

        var find = playerDao.findById(UUID.randomUUID().toString());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        PlayerEntity playerA = new PlayerEntity("PlayerA");
        PlayerEntity playerB = new PlayerEntity("PlayerB");
        playerDao.create(playerA);
        playerDao.create(playerB);
        assertThat(playerDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        PlayerEntity playerA = new PlayerEntity("PlayerA");
        playerDao.create(playerA);
        playerA.setAssistStat(3);
        playerA.setDeathStat(3);
        playerA.setKillStat(3);
        playerDao.update(playerA);
        assertThat(playerDao.findById(playerA.getId())).isEqualTo(playerA);
    }

    @Test
    void delete() {
        PlayerEntity alpha = new PlayerEntity("Alpha");
        PlayerEntity beta = new PlayerEntity("Beta");
        playerDao.create(alpha);
        playerDao.create(beta);
        playerDao.remove(alpha);
        var players = playerDao.findAll();
        assertThat(players).hasSize(1).contains(beta);
    }
}
