package cz.muni.fi.pa165.sampledata;

import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.services.player.PlayerService;
import cz.muni.fi.pa165.services.team.TeamService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;

/**
 * @author Adrián Piaček
 */
@Component
@Transactional //transactions are handled on facade layer
public class SampleDataLoadingFacadeImpl implements SampleDataLoadingFacade {

    final static Logger log = LoggerFactory.getLogger(SampleDataLoadingFacadeImpl.class);

    @Autowired
    private PlayerService playerService;

    @Autowired
    private TeamService teamService;

    @Override
    @SuppressWarnings("unused")
    public void loadData() throws IOException {


        TeamEntity g2_esports = team("g2_esports", CountryEnum.GERMANY);
        TeamEntity skt_t1 = team("skt_t1", CountryEnum.SOUTH_KOREA);

        PlayerEntity crimsix = player("9384b466-badf-40ca-bbeb-4d8f6eb98042", "Ian Porter",
                CountryEnum.AFGHANISTAN,g2_esports,5,6,7);

        PlayerEntity JJoNak = player("6e374a4e-5213-4581-847c-45f9e73ffe90", "Bang Seong-hyun ",
                CountryEnum.ALBANIA,skt_t1,5,6,7);
        log.info("Loaded Sample Players.");
    }

    private PlayerEntity player(String uuid, String name, CountryEnum country, TeamEntity team, int killStat, int deathStat, int assistStat) {
        PlayerEntity player = new PlayerEntity();
        player.setId(uuid);
        player.setName(name);
        player.setCountry(country);
        player.setTeam(team);
        player.setKillStat(killStat);
        player.setDeathStat(deathStat);
        player.setAssistStat(assistStat);
        playerService.create(player);
        return player;
    }

    private TeamEntity team(String name, CountryEnum country) {
        TeamEntity team = new TeamEntity();
        team.setName(name);
        team.setCountry(country);
        teamService.create(team);
        return team;
    }

}
