package cz.muni.fi.pa165.services.team;

import cz.muni.fi.pa165.dao.team.TeamDao;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@Service
@Transactional
public class TeamServiceImpl implements TeamService {

    @Autowired
    private TeamDao teamDao;

    @Override
    public void create(TeamEntity team) {
        if (team == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        teamDao.create(team);
    }

    @Override
    public List<TeamEntity> findAll() {
        return teamDao.findAll();
    }

    @Override
    public TeamEntity findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Given id is null.");
        }
        return teamDao.findById(id);
    }

    @Override
    public String update(TeamEntity team) {
        if (team == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        return teamDao.update(team);
    }

    @Override
    public void remove(TeamEntity team) {
        if (team == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        teamDao.remove(team);
    }

    @Override
    public void addPlayerToTheTeam(TeamEntity team, PlayerEntity newPlayer) {
        if (team == null || newPlayer == null) {
            throw new IllegalArgumentException("Given argument(s) is null.");
        }

        team.addPlayerToTheTeam(newPlayer);
    }

    @Override
    public void removePlayerFromTheTeam(TeamEntity team, PlayerEntity oldPlayer) {
        if (team == null || oldPlayer == null) {
            throw new IllegalArgumentException("Given argument(s) is null.");
        }

        team.removePlayerFromTheTeam(oldPlayer);
    }

    @Override
    public void addCoachToTheTeam(TeamEntity team, TeamCoachEntity newCoach) {
        if (team == null || newCoach == null) {
            throw new IllegalArgumentException("Given argument(s) is null.");
        }

        team.addCoachToTheTeam(newCoach);
    }

    @Override
    public void removeCoachFromTheTeam(TeamEntity team, TeamCoachEntity oldCoach) {
        if (team == null || oldCoach == null) {
            throw new IllegalArgumentException("Given argument(s) is null.");
        }

        team.removeCoachFromTheTeam(oldCoach);
    }

}
