package cz.muni.fi.pa165.services.tournament;

import cz.muni.fi.pa165.dao.tournament.TournamentDao;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@Service
@Transactional
public class TournamentServiceImpl implements TournamentService {

    @Autowired
    private TournamentDao tournamentDao;

    @Override
    public List<TournamentEntity> getTournamentsByCity(String cityName) {
        if (cityName == null) {
            throw new IllegalArgumentException("Given city name parameter is null.");
        }
        return tournamentDao.getTournamentsByCity(cityName);
    }

    @Override
    public List<TournamentEntity> getTournamentsWonBy(TeamEntity team) {
        if (team == null) {
            throw new IllegalArgumentException("Given team is null.");
        }
        return tournamentDao.getTournamentsWonBy(team);
    }

    @Override
    public void create(TournamentEntity tournament) {
        if (tournament == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        tournamentDao.create(tournament);
    }

    @Override
    public List<TournamentEntity> findAll() {
        return tournamentDao.findAll();
    }

    @Override
    public TournamentEntity findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Given id is null.");
        }
        return tournamentDao.findById(id);
    }

    @Override
    public String update(TournamentEntity tournament) {
        if (tournament == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        return tournamentDao.update(tournament);
    }

    @Override
    public void remove(TournamentEntity tournament) {
        if (tournament == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        tournamentDao.remove(tournament);
    }

    @Override
    public boolean addTeamToTournament(TournamentEntity tournament, TeamEntity team) {
        if (tournament == null || team == null) {
            throw new IllegalArgumentException("Given argument(s) is null.");
        }
        return tournament.addTeamToTournament(team);
    }

    @Override
    public void signTeamForTournament(TournamentEntity tournament, TeamEntity team) {
        if (tournament == null || team == null) {
            throw new IllegalArgumentException("Given argument(s) is null.");
        }
        tournament.signTeamForTournament(team);
    }
}
