package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.person.PersonDao;
import cz.muni.fi.pa165.entity.PersonEntity;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.services.person.PersonService;
import cz.muni.fi.pa165.services.person.PersonServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class PersonServiceTest {

    @Mock
    private PersonDao personDao;

    @Autowired
    @InjectMocks
    private PersonService personService = new PersonServiceImpl();

    private PersonEntity personA;

    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestPerson() {
        this.personA = new PersonEntity("Personalna Osoba");
        personA.setId(UUID.randomUUID().toString());
    }

    @Test
    public void createPerson() {
        personService.create(personA);

        verify(personDao, times(1)).create(personA);
    }

    @Test
    public void createNullPerson() {
        Assert.assertThrows(IllegalArgumentException.class, () -> personService.create(null));
    }

    @Test
    public void findPersonById() {
        when(personDao.findById(personA.getId())).thenReturn(personA);

        PersonEntity result = personService.findById(personA.getId());

        Assert.assertEquals(result.getName(), personA.getName());
    }

    @Test
    public void findPersonByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> personService.findById(null));
    }


    @Test
    public void findAllPersons() {
        PersonEntity personB = new PersonEntity("Autista Osobny");
        when(personDao.findAll()).thenReturn(asList(personB, personA));

        List<PersonEntity> result = personService.findAll();

        Assert.assertEquals(result.size(), 2);
    }

    @Test
    public void deletePerson() {
        personService.remove(personA);

        verify(personDao, times(1)).remove(personA);
    }

    @Test
    public void deleteNullPerson() {
        Assert.assertThrows(IllegalArgumentException.class, () -> personService.remove(null));
    }

    @Test
    public void updatePerson() {
        PlayerEntity playerB = new PlayerEntity("Autista");
        personService.update(playerB);

        verify(personDao, times(1)).update(playerB);
    }

    @Test
    public void updateNullPerson() {
        Assert.assertThrows(IllegalArgumentException.class, () -> personService.update(null));
    }
}
