package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.player.PlayerDao;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.services.player.PlayerService;
import cz.muni.fi.pa165.services.player.PlayerServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek, Marek Kadlencik
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class PlayerServiceTest {
    @Mock
    private PlayerDao playerDao;

    @Autowired
    @InjectMocks
    private PlayerService playerService = new PlayerServiceImpl();

    private PlayerEntity playerA;

    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestPlayer() {
        this.playerA = new PlayerEntity("Dummy");
        playerA.setId(UUID.randomUUID().toString());
    }

    @Test
    public void createPlayer() {
        playerService.create(playerA);

        verify(playerDao, times(1)).create(playerA);
    }

    @Test
    public void createNullPlayer() {
        Assert.assertThrows(IllegalArgumentException.class, () -> playerService.create(null));
    }

    @Test
    public void findPlayerById() {
        when(playerDao.findById(playerA.getId())).thenReturn(playerA);

        PlayerEntity result = playerService.findById(playerA.getId());

        Assert.assertEquals(result.getName(), playerA.getName());
    }

    @Test
    public void findPlayerByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> playerService.findById(null));
    }


    @Test
    public void findAllPlayers() {
        PlayerEntity playerB = new PlayerEntity();
        when(playerDao.findAll()).thenReturn(asList(playerB, playerA));

        List<PlayerEntity> result = playerService.findAll();

        Assert.assertEquals(result.size(), 2);
    }

    @Test
    public void deletePlayer() {
        playerService.remove(playerA);

        verify(playerDao, times(1)).remove(playerA);
    }

    @Test
    public void deleteNullPlayer() {
        Assert.assertThrows(IllegalArgumentException.class, () -> playerService.remove(null));
    }

    @Test
    public void updatePlayer() {
        PlayerEntity playerB = new PlayerEntity("Autista");
        playerService.update(playerB);

        verify(playerDao, times(1)).update(playerB);
    }

    @Test
    public void updateNullPlayer() {
        Assert.assertThrows(IllegalArgumentException.class, () -> playerService.update(null));
    }

}
