package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.tournamentOrganizer.TournamentOrganizerDao;
import cz.muni.fi.pa165.entity.TournamentOrganizerEntity;
import cz.muni.fi.pa165.services.tournamentOrganizer.TournamentOrganizerService;
import cz.muni.fi.pa165.services.tournamentOrganizer.TournamentOrganizerServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek, Marek Kadlencik
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class TournamentOrganizerServiceTest {

    @Mock
    private TournamentOrganizerDao tournamentOrganizerDao;

    @Autowired
    @InjectMocks
    private TournamentOrganizerService tournamentOrganizerService = new TournamentOrganizerServiceImpl();

    private TournamentOrganizerEntity tournamentOrganizerA;


    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestTournamentOrganizer() {
        tournamentOrganizerA = new TournamentOrganizerEntity("Autista Organizer");
        tournamentOrganizerA.setId(UUID.randomUUID().toString());
    }


    @Test
    public void createTournamentOrganizer() {
        tournamentOrganizerService.create(tournamentOrganizerA);
        Mockito.verify(tournamentOrganizerDao, Mockito.times(1)).create(tournamentOrganizerA);
    }

    @Test
    public void createNullTournamentOrganizer() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentOrganizerService.create(null));
    }

    @Test
    public void findTournamentOrganizerById() {
        when(tournamentOrganizerDao.findById(tournamentOrganizerA.getId())).thenReturn(tournamentOrganizerA);

        TournamentOrganizerEntity result = tournamentOrganizerService.findById(tournamentOrganizerA.getId());

        Assert.assertEquals(result.getId(), tournamentOrganizerA.getId());
    }

    @Test
    public void findTournamentOrganizerByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentOrganizerService.findById(null));
    }


    @Test
    public void findAllTournamentOrganizers() {
        TournamentOrganizerEntity tournamentOrganizerB = new TournamentOrganizerEntity("Autista DeOrganizer");
        when(tournamentOrganizerDao.findAll()).thenReturn(asList(tournamentOrganizerB, tournamentOrganizerA));

        List<TournamentOrganizerEntity> result = tournamentOrganizerService.findAll();

        Assert.assertEquals(result.size(), 2);
    }

    @Test
    public void deleteTournamentOrganizer() {
        tournamentOrganizerService.remove(tournamentOrganizerA);

        verify(tournamentOrganizerDao, times(1)).remove(tournamentOrganizerA);
    }

    @Test
    public void deleteNullTournamentOrganizer() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentOrganizerService.remove(null));
    }

    @Test
    public void updateTournamentOrganizer() {
        TournamentOrganizerEntity tournamentOrganizerB = new TournamentOrganizerEntity("Autista DeOrganizer");
        tournamentOrganizerService.update(tournamentOrganizerB);

        verify(tournamentOrganizerDao, times(1)).update(tournamentOrganizerB);
    }

    @Test
    public void updateNullTournamentOrganizer() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentOrganizerService.update(null));
    }

}
