package cz.muni.fi.pa165.mvc.config;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.support.ResourceBundleMessageSource;
import org.springframework.web.servlet.ViewResolver;
import org.springframework.web.servlet.config.annotation.DefaultServletHandlerConfigurer;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.view.InternalResourceViewResolver;

/**
 * @author Dominik Kozubík
 */
@EnableWebMvc
@Configuration
@Import({ServiceConfiguration.class})
@ComponentScan(basePackages = { "cz.muni.fi.pa165.mvc.controllers" })
public class MyConfig implements WebMvcConfigurer {

    private static final String TEXTS = "Texts";

    @Override
    public void addViewControllers(ViewControllerRegistry registry) {
        registry.addViewController("/").setViewName("index");
    }

    @Override
    public void configureDefaultServletHandling(DefaultServletHandlerConfigurer configurer) {
      //  log.debug("enabling default servlet for static files");
        configurer.enable();
    }

    @Bean
    public ViewResolver viewResolver() {
        //log.debug("registering JSP in /WEB-INF/jsp/ as views");
        InternalResourceViewResolver viewResolver = new InternalResourceViewResolver();
        viewResolver.setPrefix("/WEB-INF/jsp/");
        viewResolver.setSuffix(".jsp");
        return viewResolver;
    }
    @Bean
    public MessageSource messageSource() {
        //log.debug("registering ResourceBundle 'Texts' for messages");
        ResourceBundleMessageSource messageSource = new ResourceBundleMessageSource();
        // This line is important! see https://en.wikipedia.org/wiki/.properties#:~:text=In%20Java%209%20and%20newer,to%20ISO%2D8859%2D1.
        // It should not be necessary in the newer Java versions but for some reason even when converted to UTF-8, it still loads
        // the Texts*.properties as ISO-8859-1
        messageSource.setDefaultEncoding("UTF-8");
        messageSource.setBasename(TEXTS);
        return messageSource;
    }

}
